@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2022 Sean Davis <sean@bluesabre.org>
+#   Copyright (C) 2012-2023 Sean Davis <sean@bluesabre.org>
 #   Copyright (C) 2016-2018 OmegaPhil <OmegaPhil@startmail.com>
 #
 #   Portions of this file are adapted from Alacarte Menu Editor,
@@ -24,15 +24,13 @@ import os
 import xml.dom.minidom
 import xml.parsers.expat
 from locale import gettext as _
-from xml.sax.saxutils import escape
 
 import logging
-logger = logging.getLogger('menulibre') # noqa
+logger = logging.getLogger('menulibre')  # noqa
 
 import gi
 gi.require_version('GMenu', '3.0')  # noqa
-
-from gi.repository import GdkPixbuf, Gio, GLib, GMenu, Gtk
+from gi.repository import Gio, GMenu, Gtk  # type: ignore
 
 from . import util
 from .util import MenuItemTypes, escapeText, mapDesktopEnvironmentDirectories, unmapDesktopEnvironmentDirectories
@@ -41,19 +39,68 @@ locale.textdomain('menulibre')
 
 icon_theme = Gtk.IconTheme.get_default()
 
+
+def get_icon_theme_name_from_settings():
+    try:
+        settings = Gtk.Settings.get_default()
+        if settings is None:
+            return None
+        theme_name = settings.get_property("gtk-icon-theme-name")
+        if theme_name is not None and len(theme_name) > 0:
+            return theme_name
+    except BaseException:
+        pass
+    return None
+
+
+def get_icon_theme_name_from_icons():
+    try:
+        lookup = icon_theme.lookup_icon("folder", 16, 0)  # type: ignore
+        if lookup is None:
+            return None
+        filename = lookup.get_filename()
+        if filename is None:
+            return None
+        filename = os.path.realpath(filename)
+        path = filename.split('/')
+        for i in range(len(path) - 1):
+            index = os.path.join('/'.join(path[0:-i]), 'index.theme')
+            if os.path.exists(index):
+                return path[-i - 1]
+    except BaseException:
+        pass
+    return None
+
+
+def get_icon_theme_name():
+    theme_name = get_icon_theme_name_from_settings()
+    if theme_name is not None:
+        return theme_name
+
+    theme_name = get_icon_theme_name_from_icons()
+    if theme_name is not None:
+        return theme_name
+
+    return "Adwaita"
+
+
+icon_theme_name = get_icon_theme_name()
+
+
 menu_name = ""
 
 
 COL_NAME = 0
-COL_COMMENT = 1
-COL_EXEC = 2
-COL_CATEGORIES = 3
-COL_TYPE = 4
-COL_G_ICON = 5
-COL_ICON_NAME = 6
-COL_FILENAME = 7
-COL_EXPAND = 8
-COL_SHOW = 9
+COL_DISPLAY_NAME = 1
+COL_COMMENT = 2
+COL_EXEC = 3
+COL_CATEGORIES = 4
+COL_TYPE = 5
+COL_G_ICON = 6
+COL_ICON_NAME = 7
+COL_FILENAME = 8
+COL_EXPAND = 9
+COL_SHOW = 10
 
 
 def get_default_menu():
@@ -74,67 +121,38 @@ def get_default_menu():
     return None
 
 
-def load_fallback_icon(icon_size):
-    """If icon loading fails, load a fallback icon instead."""
-    info = icon_theme.lookup_icon(
-        "image-missing", icon_size,
-        Gtk.IconLookupFlags.GENERIC_FALLBACK | Gtk.IconLookupFlags.USE_BUILTIN)
-    return info.load_icon()
-
-
-def load_icon(gicon, icon_size):
-    """Load an icon, either from the icon theme or from a filename."""
-    pixbuf = None
-
-    if gicon is None:
-        return None
-
-    else:
-        info = icon_theme.lookup_by_gicon(gicon, icon_size, 0)
-
-        if info is None:
-            pixbuf = load_fallback_icon(icon_size)
-        else:
-            try:
-                pixbuf = info.load_icon()
-            except GLib.GError:
-                pixbuf = load_fallback_icon(icon_size)
-
-    if pixbuf.get_width() != icon_size or pixbuf.get_height() != icon_size:
-        pixbuf = pixbuf.scale_simple(
-            icon_size, icon_size, GdkPixbuf.InterpType.HYPER)
-    return pixbuf
-
-
 def menu_to_treestore(treestore, parent, menu_items):
     """Convert the Alacarte menu to a standard treestore."""
     for item in menu_items:
         item_type = item[0]
-        if item_type == MenuItemTypes.SEPARATOR:
+        if item_type == MenuItemTypes.SEPARATOR:  # type: ignore
             executable = ""
-            displayed_name = "<s>                    </s>"
+            name = _("Separator")
+            displayed_name = name
             # Translators: Separator menu item
             tooltip = _("Separator")
             categories = ""
             filename = None
-            icon = None
-            icon_name = ""
-            show = True
+            icon_name = "content-loading-symbolic"
+            icon = Gio.ThemedIcon.new(icon_name)
+            item_type = MenuItemTypes.SEPARATOR  # type: ignore
+            show = False
         else:
             executable = item[2]['executable']
-            displayed_name = escape(item[2]['display_name'])
+            name = item[2]['display_name']
+            displayed_name = escapeText(name)
             show = item[2]['show']
-            tooltip = escapeText(item[2]['comment'])
+            tooltip = item[2]['comment']
             categories = item[2]['categories']
             icon = item[2]['icon']
             filename = item[2]['filename']
             icon_name = item[2]['icon_name']
 
         treeiter = treestore.append(
-            parent, [displayed_name, tooltip, executable, categories,
+            parent, [name, displayed_name, tooltip, executable, categories,
                      item_type, icon, icon_name, filename, False, show])
 
-        if item_type == MenuItemTypes.DIRECTORY:
+        if item_type == MenuItemTypes.DIRECTORY:  # type: ignore
             treestore = menu_to_treestore(treestore, treeiter, item[3])
 
     return treestore
@@ -142,16 +160,17 @@ def menu_to_treestore(treestore, parent, menu_items):
 
 def get_treestore():
     """Get the TreeStore implementation of the current menu."""
-    treestore = Gtk.TreeStore(
-        str, # Name
-        str, # Comment
-        str, # Executable
-        str, # Categories
-        int, # MenuItemType
-        Gio.Icon, # GIcon
-        str, # icon-name
-        str, # Filename
-        bool, # Expand
+    treestore = Gtk.TreeStore(  # type: ignore
+        str,  # Name
+        str,  # Displayed Name
+        str,  # Comment
+        str,  # Executable
+        str,  # Categories
+        int,  # MenuItemType
+        Gio.Icon,  # GIcon
+        str,  # icon-name
+        str,  # Filename
+        bool,  # Expand
         bool  # Show
     )
     menus = get_menus()
@@ -160,19 +179,42 @@ def get_treestore():
     return menu_to_treestore(treestore, None, menus[0])
 
 
+def get_extended_icons(icon_names, extended_names):
+    results = []
+    prefer_symbolic = icon_theme_name in [
+        "Adwaita"] and "folder" in extended_names
+    for name in icon_names:
+        if icon_theme.has_icon(name):
+            return results
+    if prefer_symbolic:
+        for name in extended_names:
+            name = name + "-symbolic"
+            if icon_theme.has_icon(name):
+                results.append(name)
+        if len(results) > 0:
+            return results
+    for name in extended_names:
+        if icon_theme.has_icon(name):
+            results.append(name)
+    return results
+
+
 def get_submenus(menu, tree_dir):
     """Get the submenus for a tree directory."""
     structure = []
     for child in menu.getContents(tree_dir):
         if isinstance(child, GMenu.TreeSeparator):
-            structure.append([MenuItemTypes.SEPARATOR, child, None, None])
+            structure.append([MenuItemTypes.SEPARATOR, child, None, None])  # type: ignore
         else:
             if isinstance(child, GMenu.TreeEntry):
-                item_type = MenuItemTypes.APPLICATION
+                item_type = MenuItemTypes.APPLICATION  # type: ignore
                 entry_id = child.get_desktop_file_id()
                 app_info = child.get_app_info()
                 icon = app_info.get_icon()
-                icon_name = "applications-other"
+                icon_name = app_info.get_string("Icon")
+                extended_icon_names = [
+                    "applications-other",
+                    "application-x-executable"]
                 display_name = app_info.get_display_name()
                 generic_name = app_info.get_generic_name()
                 comment = app_info.get_description()
@@ -188,10 +230,11 @@ def get_submenus(menu, tree_dir):
                 hidden = is_hidden or no_display or not show_in
 
             elif isinstance(child, GMenu.TreeDirectory):
-                item_type = MenuItemTypes.DIRECTORY
+                item_type = MenuItemTypes.DIRECTORY  # type: ignore
                 entry_id = child.get_menu_id()
                 icon = child.get_icon()
-                icon_name = "folder"
+                icon_name = None
+                extended_icon_names = ["folder"]
                 display_name = child.get_name()
                 generic_name = child.get_generic_name()
                 comment = child.get_comment()
@@ -202,12 +245,32 @@ def get_submenus(menu, tree_dir):
                 hidden = child.get_is_nodisplay()
                 submenus = get_submenus(menu, child)
 
+            else:
+                continue
+
+            icon_names = []
             if isinstance(icon, Gio.ThemedIcon):
-                icon_name = icon.get_names()[0]
+                icon_names = icon.get_names()
             elif isinstance(icon, Gio.FileIcon):
-                icon_name = icon.get_file().get_path()
-            
-            filename = os.path.realpath(filename)
+                icon_names = [icon.get_file().get_path()]
+
+            if icon_name is not None:
+                icon_names = [icon_name] + icon_names
+
+            for extended_icon_name in get_extended_icons(
+                    icon_names, extended_icon_names):
+                if isinstance(icon, Gio.ThemedIcon):
+                    icon.append_name(extended_icon_name)
+                icon_names.append(extended_icon_name)
+
+            if icon_name is None:
+                icon_name = icon_names[0]
+
+            elif icon is None:
+                icon = Gio.ThemedIcon.new(icon_name)
+
+            if filename is not None:
+                filename = os.path.realpath(filename)
 
             details = {'display_name': display_name,
                        'generic_name': generic_name,
@@ -259,13 +322,14 @@ def removeWhitespaceNodes(node):
 def getUserMenuXml(tree):
     """Return the header portions of the menu xml file."""
     system_file = util.getSystemMenuPath(
-            os.path.basename(tree.get_canonical_menu_path()))
+        os.path.basename(tree.get_canonical_menu_path()))
     name = tree.get_root_directory().get_menu_id()
     menu_xml = "<!DOCTYPE Menu PUBLIC '-//freedesktop//DTD Menu 1.0//EN'" \
                " 'http://standards.freedesktop.org/menu-spec/menu-1.0.dtd'>\n"
     menu_xml += "<Menu>\n  <Name>" + name + "</Name>\n  "
-    menu_xml += "<MergeFile type=\"parent\">" + system_file + \
-                "</MergeFile>\n</Menu>\n"
+    if system_file is not None:
+        menu_xml += "<MergeFile type=\"parent\">" + system_file + "</MergeFile>\n"
+    menu_xml += "</Menu>\n"
     return menu_xml
 
 
@@ -317,7 +381,7 @@ class MenuEditor(object):
         if not self.tree.load_sync():
             raise ValueError("can not load menu tree %r" %
                              (self.tree.props.menu_basename,))
-    
+
     def unmap(self):
         unmapDesktopEnvironmentDirectories()
 
@@ -339,7 +403,7 @@ class MenuEditor(object):
         while item_type != GMenu.TreeItemType.INVALID:
             if item_type == GMenu.TreeItemType.DIRECTORY:
                 item = item_iter.get_directory()
-                yield (item, self.isVisible(item))
+                yield (item, True)
             item_type = item_iter.next()
 
     def getContents(self, item):
