@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2022 Sean Davis <sean@bluesabre.org>
+#   Copyright (C) 2012-2023 Sean Davis <sean@bluesabre.org>
 #
 #   This program is free software: you can redistribute it and/or modify it
 #   under the terms of the GNU General Public License version 3, as published
@@ -15,7 +15,7 @@
 #   You should have received a copy of the GNU General Public License along
 #   with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import GObject
+from gi.repository import GObject  # type: ignore
 
 import logging
 logger = logging.getLogger('menulibre')
@@ -35,7 +35,7 @@ class History(GObject.GObject):
     }
 
     def __init__(self):
-        """Intialize the MenulibreHistory object."""
+        """Initialize the MenulibreHistory object."""
         GObject.GObject.__init__(self)
         self._undo = []
         self._redo = []
