@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2022 Sean Davis <sean@bluesabre.org>
+#   Copyright (C) 2012-2023 Sean Davis <sean@bluesabre.org>
 #   Copyright (C) 2016 OmegaPhil <OmegaPhil@startmail.com>
 #
 #   This program is free software: you can redistribute it and/or modify it
@@ -21,7 +21,7 @@ import subprocess
 
 from locale import gettext as _
 
-from gi.repository import Gio, GObject, Gtk, Pango, GLib
+from gi.repository import Gio, GObject, Gtk, Pango, GLib  # type: ignore
 
 from . import MenuEditor, MenulibreXdg, XmlMenuElementTree, util
 from .util import MenuItemTypes, check_keypress, getBasename, getRelativeName, escapeText
@@ -30,7 +30,7 @@ import logging
 logger = logging.getLogger('menulibre')
 
 
-class Treeview(GObject.GObject):
+class Treeview(Gtk.Box):
 
     __gsignals__ = {
         'cursor-changed': (GObject.SIGNAL_RUN_LAST, GObject.TYPE_BOOLEAN,
@@ -39,54 +39,55 @@ class Treeview(GObject.GObject):
                                   GObject.TYPE_BOOLEAN,
                                   (GObject.TYPE_BOOLEAN,)),
         'requires-menu-reload': (GObject.SIGNAL_RUN_LAST,
-                                  GObject.TYPE_BOOLEAN,
-                                  (GObject.TYPE_BOOLEAN,)),
+                                 GObject.TYPE_BOOLEAN,
+                                 (GObject.TYPE_BOOLEAN,)),
     }
 
     loaded = False
 
-    def __init__(self, parent, builder):
-        GObject.GObject.__init__(self)
+    def __init__(self, parent):
+        super().__init__(orientation=Gtk.Orientation.VERTICAL, spacing=0)
         self.parent = parent
-
-        # Configure Widgets
-        if self._configure_treeview(builder):
-            self.loaded = True
-        else:
-            return
-        self._configure_toolbar(builder)
-
-        # Defaults
         self._last_selected_path = -1
         self._search_terms = None
         self._lock_menus = False
 
-    def _configure_treeview(self, builder):
-        """Configure the TreeView widget."""
-        # Get the menu treestore.
-        treestore = MenuEditor.get_treestore()
-        if not treestore:
-            return False
+        self.set_size_request(220, -1)
 
-        self._treeview = builder.get_object('classic_view_treeview')
+        scrolled = Gtk.ScrolledWindow.new(hadjustment=None, vadjustment=None)
+        scrolled.set_shadow_type(Gtk.ShadowType.IN)
+        scrolled.set_name("MenulibreSidebarScroll")
+        self.pack_start(scrolled, True, True, 0)
+
+        treestore = MenuEditor.get_treestore()
+        if treestore is not None:
+            self.loaded = True
+            self._treeview = Gtk.TreeView.new_with_model(treestore)
+        else:
+            self._treeview = Gtk.TreeView.new()
+
+        self._treeview.set_show_expanders(True)
+        self._treeview.set_enable_search(False)
+        self._treeview.set_headers_visible(False)
+        scrolled.add(self._treeview)
 
         # Translators: "Search Results" treeview column header
-        col = Gtk.TreeViewColumn(_("Search Results"))
+        col = Gtk.TreeViewColumn(_("Search Results"))  # type: ignore
 
-        # Create and pack the PixbufRenderer.
+        # Icon renderer
         col_cell_img = Gtk.CellRendererPixbuf()
         col_cell_img.set_property("stock-size", Gtk.IconSize.LARGE_TOOLBAR)
         col.pack_start(col_cell_img, False)
 
-        # Create and pack the TextRenderer.
+        # Text renderer
         col_cell_text = Gtk.CellRendererText()
         col_cell_text.set_property("ellipsize", Pango.EllipsizeMode.END)
         col.pack_start(col_cell_text, True)
 
         # Set the markup property on the Text cell.
-        col.add_attribute(col_cell_text, "markup", 0)
+        col.add_attribute(col_cell_text, "markup", MenuEditor.COL_DISPLAY_NAME)
 
-        # Add the cell data func for the pixbuf column to render icons.
+        # Add the cell data func for the text column to render labels.
         col.set_cell_data_func(col_cell_text, self._text_display_func, None)
 
         # Set the Tooltip column.
@@ -97,11 +98,10 @@ class Treeview(GObject.GObject):
 
         # Append the column, set the model.
         self._treeview.append_column(col)
-        self._treeview.set_model(treestore)
 
         # Configure the treeview events.
         self._treeview.connect("cursor-changed",
-                               self._on_treeview_cursor_changed, None, builder)
+                               self._on_treeview_cursor_changed, None)
         self._treeview.connect("key-press-event",
                                self._on_treeview_key_press_event, None)
         self._treeview.connect("row-expanded",
@@ -109,26 +109,41 @@ class Treeview(GObject.GObject):
         self._treeview.connect("row-collapsed",
                                self._on_treeview_row_expansion, False)
 
-        # Show the treeview, grab focus.
-        self._treeview.show_all()
-        self._treeview.grab_focus()
-
         self.menu_timeout_id = 0
 
-        return True
+        self._toolbar = Gtk.Toolbar.new()
+        self._toolbar.set_icon_size(Gtk.IconSize.MENU)
+        self._toolbar.set_name("MenulibreSidebarToolbar")
+        self.add(self._toolbar)
 
-    def _configure_toolbar(self, builder):
-        """Configure the toolbar widget."""
-        self._toolbar = builder.get_object('browser_toolbar')
-        move_up = builder.get_object('classic_view_move_up')
-        move_up.connect('clicked', self._move_iter, (self._treeview, -1))
-        move_down = builder.get_object('classic_view_move_down')
-        move_down.connect('clicked', self._move_iter, (self._treeview, 1))
-        sort = builder.get_object('classic_view_sort')
-        sort.connect('clicked', self._sort_iter)
-        self._sort_button = sort
-        self._move_up_button = move_up
-        self._move_down_button = move_down
+        context = self._toolbar.get_style_context()
+        context.add_class("inline-toolbar")
+
+        img = Gtk.Image.new_from_icon_name('go-up-symbolic', Gtk.IconSize.MENU)
+        self._move_up_button = Gtk.ToolButton.new(img, _("Move Up"))
+        self._move_up_button.set_tooltip_text(_("Move Up"))
+        self._move_up_button.connect(
+            'clicked', self._move_iter, (self._treeview, -1))
+        self._toolbar.add(self._move_up_button)
+
+        img = Gtk.Image.new_from_icon_name(
+            'go-down-symbolic', Gtk.IconSize.MENU)
+        self._move_down_button = Gtk.ToolButton.new(img, _("Move Down"))
+        self._move_down_button.set_tooltip_text(_("Move Down"))
+        self._move_down_button.connect(
+            'clicked', self._move_iter, (self._treeview, 1))
+        self._toolbar.add(self._move_down_button)
+
+        img = Gtk.Image.new_from_icon_name(
+            'view-sort-ascending-symbolic', Gtk.IconSize.MENU)
+        self._sort_button = Gtk.ToolButton.new(img, _("Sort Alphabetically"))
+        self._sort_button.set_tooltip_text(_("Sort Alphabetically"))
+        self._sort_button.connect('clicked', self._sort_iter)
+        self._toolbar.add(self._sort_button)
+
+        # Show the treeview, grab focus.
+        self.show_all()
+        self._treeview.grab_focus()
 
     def set_sortable(self, sortable):
         self._sort_button.set_sensitive(sortable)
@@ -167,6 +182,9 @@ class Treeview(GObject.GObject):
         if treeiter is None or model is None:
             model, treeiter = self._get_selected_iter()
 
+        if treeiter is None:
+            return None
+
         new_iter = model.prepend(treeiter)
 
         if do_select:
@@ -184,10 +202,17 @@ class Treeview(GObject.GObject):
         self._last_selected_path = -1
         model, treeiter = self._get_selected_iter()
 
+        filename = None
+        original = None
+        item_type = None
+        del_files = []
+
         if not ui_only:
 
-            filename = model[treeiter][MenuEditor.COL_FILENAME]
-            item_type = model[treeiter][MenuEditor.COL_TYPE]
+            if treeiter is not None:
+                filename = model[treeiter][MenuEditor.COL_FILENAME]
+                item_type = model[treeiter][MenuEditor.COL_TYPE]
+
             if filename is not None:
                 basename = getRelativeName(filename)
                 original = util.getSystemLauncherPath(basename)
@@ -217,11 +242,12 @@ class Treeview(GObject.GObject):
 
             # How is it possible that a launcher is to be removed, but its
             # associated desktop file has not been returned to be deleted?
-            if filename not in del_files:
+            if filename is None or filename not in del_files:
                 # Update the required categories.
                 model, parent_data = self.get_parent_row_data()
                 if parent_data is not None:
-                    categories = util.getRequiredCategories(parent_data[MenuEditor.COL_FILENAME])
+                    categories = util.getRequiredCategories(
+                        parent_data[MenuEditor.COL_FILENAME])
                 else:
                     categories = util.getRequiredCategories(None)
                 self.parent.update_launcher_categories(categories, [])
@@ -237,8 +263,15 @@ class Treeview(GObject.GObject):
                 executable = entry['Exec']
                 icon_name = entry['Icon']
                 hidden = entry['Hidden'] or entry['NoDisplay']
-                self.update_selected(name, comment, executable, categories,
-                                     item_type, icon_name, original, not hidden)
+                self.update_selected(
+                    name,
+                    comment,
+                    executable,
+                    categories,
+                    item_type,
+                    icon_name,
+                    original,
+                    not hidden)
                 model, row_data = self.get_selected_row_data()
                 self.update_launcher_instances(filename, row_data)
                 treeiter = None
@@ -268,7 +301,7 @@ class Treeview(GObject.GObject):
         parent = None
         if model is None:
             model, treeiter = self._get_selected_iter()
-        if treeiter:
+        if treeiter is not None:
             path = model.get_path(treeiter)
             if path.up():
                 if path.get_depth() > 0:
@@ -292,7 +325,7 @@ class Treeview(GObject.GObject):
         try:
             string = path.to_string()
             parts = string.split(":")
-            parts[-1] = str(int(parts[-1])+1)
+            parts[-1] = str(int(parts[-1]) + 1)
             string = ":".join(parts)
             path = Gtk.TreePath.new_from_string(string)
             model.get_iter(path)
@@ -320,9 +353,10 @@ class Treeview(GObject.GObject):
     def get_parent_row_data(self):
         """Get the row data of the parent iter."""
         model, parent = self.get_parent()
-        if parent is not None:
-            return model, model[parent][:]
-        return model, None
+        if parent is None:
+            return model, None
+        parent_data: list = model[parent][:]  # type: ignore
+        return model, parent_data
 
     def get_selected_filename(self):
         """Return the filename of the current selected treeiter."""
@@ -336,7 +370,8 @@ class Treeview(GObject.GObject):
         model, treeiter = self._get_selected_iter()
         if model is None or treeiter is None:
             return model, None
-        return model, model[treeiter][:]
+        row_data: list = model[treeiter][:]  # type: ignore
+        return model, row_data
 
 # Set
     def set_can_select_function(self, can_select_func):
@@ -357,8 +392,12 @@ class Treeview(GObject.GObject):
                         icon_name, filename, show=True):
         """Update the application treeview selected row data."""
         model, treeiter = self._get_selected_iter()
+        if treeiter is None:
+            return
+
         model[treeiter][MenuEditor.COL_NAME] = name
-        model[treeiter][MenuEditor.COL_COMMENT] = escapeText(comment)
+        model[treeiter][MenuEditor.COL_DISPLAY_NAME] = escapeText(name)
+        model[treeiter][MenuEditor.COL_COMMENT] = comment
         model[treeiter][MenuEditor.COL_EXEC] = executable
         model[treeiter][MenuEditor.COL_CATEGORIES] = categories
         model[treeiter][MenuEditor.COL_TYPE] = item_type
@@ -374,10 +413,10 @@ class Treeview(GObject.GObject):
 
         # Refresh the displayed launcher
         self._last_selected_path = -1
-        self._on_treeview_cursor_changed(self._treeview, None, None)
+        self._on_treeview_cursor_changed(self._treeview, None)
 
 # Events
-    def _on_treeview_cursor_changed(self, widget, selection, builder):
+    def _on_treeview_cursor_changed(self, widget, selection):
         """Update the editor frame when the selected row is changed."""
         # Check if the selection is valid.
         sel = widget.get_selection()
@@ -453,15 +492,21 @@ class Treeview(GObject.GObject):
             renderer.set_property("style", Pango.Style.NORMAL)
         else:
             renderer.set_property("style", Pango.Style.ITALIC)
+        separator = treestore[treeiter][MenuEditor.COL_TYPE] == MenuItemTypes.SEPARATOR  # type: ignore
+        renderer.set_property("sensitive", not separator)
         renderer.set_property("style-set", True)
 
     def _icon_name_func(self, col, renderer, treestore, treeiter, user_data):
         """CellRenderer function to set the gicon for each row."""
-        renderer.set_property("gicon", treestore[treeiter][MenuEditor.COL_G_ICON])
+        renderer.set_property("gicon",
+                              treestore[treeiter][MenuEditor.COL_G_ICON])
+        separator = treestore[treeiter][MenuEditor.COL_TYPE] == MenuItemTypes.SEPARATOR  # type: ignore
+        renderer.set_property("sensitive", not separator)
 
     def _get_selected_iter(self):
         """Return the current treeview model and selected iter."""
-        model, treeiter = self._treeview.get_selection().get_selected()
+        model: Gtk.TreeStore
+        model, treeiter = self._treeview.get_selection().get_selected()  # type: ignore
         return model, treeiter
 
     def _populate_and_select_iter(self, model, treeiter, row_data,
@@ -493,7 +538,7 @@ class Treeview(GObject.GObject):
             basename = getRelativeName(filename)
             original = util.getSystemLauncherPath(basename)
             item_type = model[treeiter][MenuEditor.COL_TYPE]
-            if original is None and item_type == MenuItemTypes.DIRECTORY:
+            if original is None and item_type == MenuItemTypes.DIRECTORY:  # type: ignore
                 pass
             else:
                 block_run = True
@@ -530,11 +575,15 @@ class Treeview(GObject.GObject):
 
     def _get_launcher_instances(self, filename, model=None, parent=None):
         """Return a list of all treeiters referencing this filename."""
+        if filename is None:
+            return []
         if model is None:
             model, treeiter = self._get_selected_iter()
         treeiters = []
         for n_child in range(model.iter_n_children(parent)):
             treeiter = model.iter_nth_child(parent, n_child)
+            if treeiter is None:
+                continue
             iter_filename = model[treeiter][MenuEditor.COL_FILENAME]
             if iter_filename == filename:
                 treeiters.append(treeiter)
@@ -550,10 +599,6 @@ class Treeview(GObject.GObject):
         """Return True if menu editing is currently locked."""
         return self._lock_menus
 
-    def get_treeview(self):
-        """Return the treeview widget."""
-        return self._treeview
-
     def _update_add_directory(self):
         """Prevent adding subdirectories to system menus."""
         add_enabled = True
@@ -573,12 +618,15 @@ class Treeview(GObject.GObject):
     def search(self, terms):
         """Search the treeview for the specified terms."""
         self._search_terms = str(terms.lower())
-        model = self._treeview.get_model()
+        model: Gtk.TreeModelFilter = self._treeview.get_model()  # type: ignore
         model.refilter()
 
     def set_searchable(self, searchable, expand=False):
         """Set the TreeView searchable."""
         model = self._treeview.get_model()
+        if model is None:
+            return
+
         if searchable:
             self._lock_menus = True
             # Show the "Search Results" header and disable the inline toolbar.
@@ -613,13 +661,15 @@ class Treeview(GObject.GObject):
                 self._treeview.collapse_all()
                 for n_child in range(model.iter_n_children(None)):
                     treeiter = model.iter_nth_child(None, n_child)
+                    if treeiter is None:
+                        continue
                     row = model[treeiter]
                     if row[MenuEditor.COL_FILENAME]:
                         self._treeview.expand_row(row.path, False)
 
                 # Try to get the row that was selected previously.
                 if (f_model is not None) and (f_iter is not None):
-                    row_data = f_model[f_iter][:]
+                    row_data = f_model[f_iter][:]  # type: ignore
                     selected_iter = self._get_iter_by_data(row_data, model,
                                                            parent=None)
 
@@ -627,17 +677,21 @@ class Treeview(GObject.GObject):
                 else:
                     selected_iter = model.get_iter_first()
 
-                # Set the cursor.
-                path = model.get_path(selected_iter)
-                self._treeview.set_cursor(path)
+                if selected_iter is not None:
+                    # Set the cursor.
+                    path = model.get_path(selected_iter)
+                    self._treeview.set_cursor(path)
 
     def _treeview_match(self, model, treeiter, query):
         """Match subfunction for filtering search results."""
-        name, comment, executable, categories, item_type, icon, pixbuf, desktop, \
-            expanded, show = model[treeiter][:]
+        name = model[treeiter][MenuEditor.COL_NAME]
+        comment = model[treeiter][MenuEditor.COL_COMMENT]
+        executable = model[treeiter][MenuEditor.COL_EXEC]
+        item_type = model[treeiter][MenuEditor.COL_TYPE]
+        desktop = model[treeiter][MenuEditor.COL_FILENAME]
 
         # Hide separators in the search results.
-        if item_type == MenuItemTypes.SEPARATOR:
+        if item_type == MenuItemTypes.SEPARATOR:  # type: ignore
             return False
 
         # Convert None to blank.
@@ -660,7 +714,7 @@ class Treeview(GObject.GObject):
             return True
 
         if query in executable.lower():
-                return True
+            return True
 
         # Match against the desktop file.
         desktop = desktop.replace("menulibre-", "")
@@ -669,7 +723,7 @@ class Treeview(GObject.GObject):
             return True
 
         # Show the directory if any child items match.
-        if item_type == MenuItemTypes.DIRECTORY:
+        if item_type == MenuItemTypes.DIRECTORY:  # type: ignore
             return self._treeview_match_directory(query, model, treeiter)
 
         # No matches, return False.
@@ -698,6 +752,8 @@ class Treeview(GObject.GObject):
         model, treeiter = self._get_selected_iter()
         if filename is None:
             return
+        if treeiter is None:
+            return
         if filename.endswith('.desktop'):
             menu_install = True
             parents = []
@@ -711,7 +767,8 @@ class Treeview(GObject.GObject):
                 parent = model.iter_parent(parent)
             parents.reverse()
             if menu_install:
-                installed = MenulibreXdg.desktop_menu_install(parents, [filename])
+                installed = MenulibreXdg.desktop_menu_install(parents, [
+                                                              filename])
                 if not installed:
                     self.emit("requires-menu-reload", True)
 
@@ -743,8 +800,8 @@ class Treeview(GObject.GObject):
         """Update the menu files."""
         if self.menu_timeout_id > 0:
             GLib.source_remove(self.menu_timeout_id)
-        self.menu_timeout_id = GLib.timeout_add_seconds(1,
-            self.update_menu_timeout)
+        self.menu_timeout_id = GLib.timeout_add_seconds(
+            1, self.update_menu_timeout)
 
     def update_menu_timeout(self):
         # Do not save menu layout if in search mode (lp #1306999)
@@ -827,7 +884,8 @@ class Treeview(GObject.GObject):
             # Get current required categories
             model, parent = self.get_parent(model, selected_iter)
             if parent:
-                categories = util.getRequiredCategories(model[parent][MenuEditor.COL_FILENAME])
+                categories = util.getRequiredCategories(
+                    model[parent][MenuEditor.COL_FILENAME])
             else:
                 categories = util.getRequiredCategories(None)
 
@@ -847,9 +905,9 @@ class Treeview(GObject.GObject):
                 sibling_type = model[sibling_iter][MenuEditor.COL_TYPE]
 
                 # Sibling Directory
-                if sibling_type == MenuItemTypes.DIRECTORY:
+                if sibling_type == MenuItemTypes.DIRECTORY:  # type: ignore
                     # Do not move directories into other directories.
-                    if selected_type == MenuItemTypes.DIRECTORY:
+                    if selected_type == MenuItemTypes.DIRECTORY:  # type: ignore
                         move_down = False
 
                     # Append or Prepend to expanded directories.
@@ -881,7 +939,8 @@ class Treeview(GObject.GObject):
             # Get new required categories
             model, parent = self.get_parent(model, selected_iter)
             if parent:
-                new_categories = util.getRequiredCategories(model[parent][MenuEditor.COL_FILENAME])
+                new_categories = util.getRequiredCategories(
+                    model[parent][MenuEditor.COL_FILENAME])
             else:
                 new_categories = util.getRequiredCategories(None)
 
@@ -907,13 +966,15 @@ class Treeview(GObject.GObject):
 
         self.emit("cursor-changed", True)
 
-    def _get_iter_by_data(self, row_data, model, parent=None):
+    def _get_iter_by_data(self, row_data, model: Gtk.TreeModel, parent=None):
         """Search the TreeModel for a row matching row_data.
 
         Return the TreeIter found or None if none found."""
         for n_child in range(model.iter_n_children(parent)):
             treeiter = model.iter_nth_child(parent, n_child)
-            if model[treeiter][:] == row_data:
+            if treeiter is None:
+                continue
+            if model[treeiter][:] == row_data:  # type: ignore
                 return treeiter
             if model.iter_n_children(treeiter) != 0:
                 value = self._get_iter_by_data(row_data, model, treeiter)
@@ -979,7 +1040,7 @@ class Treeview(GObject.GObject):
 
         # Get the current selected row
         model, sel_iter = self._get_selected_iter()
-        if sel_iter:
+        if sel_iter is not None:
 
             # Move to the parent iter - if there is no parent, it must be the
             # top level, which is ignored
@@ -990,26 +1051,33 @@ class Treeview(GObject.GObject):
                 # Deteriming list of item names
                 for i in range(model.iter_n_children(parent_iter)):
                     child_iter = model.iter_nth_child(parent_iter, i)
-                    item_names.append(model[child_iter][MenuEditor.COL_NAME])
+                    if child_iter is not None:
+                        item_names.append(model[child_iter][MenuEditor.COL_NAME])
 
                 # Applying unstable (?) case-insensitive alphabetical sort
                 item_names = sorted(item_names, key=str.lower)
 
                 for i in range(len(item_names)):
                     child_iter = model.iter_nth_child(parent_iter, i)
+                    if child_iter is None:
+                        continue
 
                     # Ignore if item is already sorted or at least has an
                     # identical title to that expected
                     if item_names[i] != model[child_iter][MenuEditor.COL_NAME]:
+                        search_iter = None
 
                         # Locating desired item in the remaining unsorted items
                         for r in range(i, len(item_names)):
                             search_iter = model.iter_nth_child(parent_iter, r)
+                            if search_iter is None:
+                                continue
                             if item_names[i] == model[search_iter][MenuEditor.COL_NAME]:
                                 break
 
-                        # Moving the found item into place
-                        model.move_before(search_iter, child_iter)
+                        if search_iter is not None:
+                            # Moving the found item into place
+                            model.move_before(search_iter, child_iter)
 
                 # Committing changes
                 self.update_menus()
@@ -1018,5 +1086,14 @@ class Treeview(GObject.GObject):
 
     def scroll_to_selection(self):
         model, sel_iter = self._get_selected_iter()
+        if sel_iter is None:
+            return
         self._treeview.scroll_to_cell(model.get_path(sel_iter), None,
-                                False, 0.0, 0.0)
\ No newline at end of file
+                                      False, 0.0, 0.0)
+
+    def set_search_entry(self, entry):
+        self._treeview.set_search_entry(entry)
+
+    def reset_cursor(self):
+        self._treeview.set_cursor(Gtk.TreePath.new_from_string("1"))
+        self._treeview.set_cursor(Gtk.TreePath.new_from_string("0"))
