@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2022 Sean Davis <sean@bluesabre.org>
+#   Copyright (C) 2012-2023 Sean Davis <sean@bluesabre.org>
 #
 #   This program is free software: you can redistribute it and/or modify it
 #   under the terms of the GNU General Public License version 3, as published
@@ -17,11 +17,12 @@
 
 import locale
 import os
+import json
 from locale import gettext as _
 
 import subprocess
 
-from gi.repository import GLib
+from gi.repository import GLib  # type: ignore
 
 locale.textdomain('menulibre')
 
@@ -42,11 +43,8 @@ class MenulibreDesktopEntry:
         else:
             self['Version'] = '1.1'
             self['Type'] = 'Application'
-            # Translators: Placeholder text for a new menu item name.
-            self['Name'] = _('New Menu Item')
-            # Translators: Placeholder text for a new menu item description.
-            self['Comment'] = _(
-                "A small descriptive blurb about this application.")
+            self['Name'] = _('New Launcher')
+            self['Comment'] = ''
             self['Icon'] = 'applications-other'
             self['Exec'] = ''
             self['Path'] = ''
@@ -100,7 +98,7 @@ class MenulibreDesktopEntry:
         elif "X-Ayatana-Desktop-Shortcuts" in self._get_keys("Desktop Entry"):
             action_key = "X-Ayatana-Desktop-Shortcuts"
         else:
-            return []
+            return json.dumps([])
 
         enabled_quicklists = self._get_string_list("Desktop Entry", action_key)
 
@@ -113,9 +111,9 @@ class MenulibreDesktopEntry:
             displayed_name = self.get_property(group, "Name")
             command = self.get_property(group, "Exec")
             enabled = name in enabled_quicklists
-            quicklists.append((name, displayed_name, command, enabled))
+            quicklists.append([enabled, name, displayed_name, command])
 
-        return quicklists
+        return json.dumps(quicklists)
 
     def _get_action_group_name(self, group):
         if group.startswith("Desktop Action "):
@@ -195,7 +193,7 @@ def desktop_menu_update():
 def desktop_menu_install(directory_files, desktop_files):
     """Install one or more applications in a submenu of the desktop menu
     system.  If multiple directory files are provided each file will represent
-    a submenu within the menu that preceeds it, creating a nested menu
+    a submenu within the menu that precedes it, creating a nested menu
     hierarchy (sub-sub-menus). The menu entries themselves will be added to
     the last submenu. """
     # Check for the minimum required arguments
@@ -210,11 +208,13 @@ def desktop_menu_install(directory_files, desktop_files):
 
     # xdg-desktop-menu doesn't behave nicely with vendor- directories
     # without vendor- applications.
-    directory_dir = os.path.join(GLib.get_user_data_dir(), "desktop-directories")
+    directory_dir = os.path.join(
+        GLib.get_user_data_dir(),
+        "desktop-directories")
     for filename in directory_files:
         if not filename.startswith(directory_dir):
             continue
-        if not "/" in filename:
+        if "/" not in filename:
             continue
         basename = os.path.basename(filename)
         relative = filename.split("desktop-directories/")[1]
@@ -231,6 +231,7 @@ def desktop_menu_install(directory_files, desktop_files):
 
     return True
 
+
 def desktop_menu_uninstall(directory_files, desktop_files):  # noqa
     """Remove applications or submenus from the desktop menu system
     previously installed with xdg-desktop-menu install."""
