@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2022 Sean Davis <sean@bluesabre.org>
+#   Copyright (C) 2012-2023 Sean Davis <sean@bluesabre.org>
 #   Copyright (C) 2016 OmegaPhil <OmegaPhil@startmail.com>
 #
 #   This program is free software: you can redistribute it and/or modify it
@@ -207,10 +207,11 @@ def model_to_xml_menus(model, model_parent=None, menu_parent=None):
         treeiter = model.iter_nth_child(model_parent, n_child)
 
         # Extract the menu item details.
-        name, comment, executable, categories, item_type, gicon, icon, desktop, expanded, \
-            show = model[treeiter][:]
+        name = model[treeiter][MenuEditor.COL_NAME]
+        item_type = model[treeiter][MenuEditor.COL_TYPE]
+        desktop = model[treeiter][MenuEditor.COL_FILENAME]
 
-        if item_type == MenuItemTypes.DIRECTORY:
+        if item_type == MenuItemTypes.DIRECTORY:  # type: ignore
             # Do not save duplicate directories.
             global processed_directories
             if desktop in processed_directories:
@@ -220,12 +221,12 @@ def model_to_xml_menus(model, model_parent=None, menu_parent=None):
             if desktop is None:
                 # Cinnamon fix.
                 if name == 'wine-wine':
-                    next_element = menu_parent.addMenu(name)
+                    next_element = menu_parent.addMenu(name)  # type: ignore
                 else:
                     continue
             else:
                 directory_name = util.getDirectoryName(desktop)
-                next_element = menu_parent.addMenu(directory_name, desktop)
+                next_element = menu_parent.addMenu(directory_name, desktop)  # type: ignore
 
             # Do Menus
             model_to_xml_menus(model, treeiter, next_element)
@@ -237,10 +238,10 @@ def model_to_xml_menus(model, model_parent=None, menu_parent=None):
             # Do Layouts
             model_to_xml_layout(model, treeiter, next_element)
 
-        elif item_type == MenuItemTypes.APPLICATION:
+        elif item_type == MenuItemTypes.APPLICATION:  # type: ignore
             pass
 
-        elif item_type == MenuItemTypes.SEPARATOR:
+        elif item_type == MenuItemTypes.SEPARATOR:  # type: ignore
             pass
 
 
@@ -254,8 +255,9 @@ def model_to_xml_includes(model, model_parent=None, menu_parent=None):
         treeiter = model.iter_nth_child(model_parent, n_child)
 
         # Extract the menu item details.
-        name, comment, executable, categories, item_type, gicon, icon, desktop, expanded, \
-            show = model[treeiter][:]
+        categories = model[treeiter][MenuEditor.COL_CATEGORIES]
+        item_type = model[treeiter][MenuEditor.COL_TYPE]
+        desktop = model[treeiter][MenuEditor.COL_FILENAME]
 
         if desktop is None:
             continue
@@ -270,9 +272,9 @@ def model_to_xml_includes(model, model_parent=None, menu_parent=None):
 
         # Items in custom directories by menulibre have a category, but
         # includes are required otherwise they are dropped by GMenu
-        if item_type == MenuItemTypes.APPLICATION and (
+        if item_type == MenuItemTypes.APPLICATION and (  # type: ignore
                 not categories or user_directory):
-            include = menu_parent.addInclude()
+            include = menu_parent.addInclude()  # type: ignore
             try:
                 include.addFilename(os.path.basename(desktop))
             except AttributeError:
@@ -282,7 +284,7 @@ def model_to_xml_includes(model, model_parent=None, menu_parent=None):
 def model_to_xml_layout(model, model_parent=None, menu_parent=None,  # noqa
                         merge=True):
     """Append the <Layout> element to menu_parent."""
-    layout = menu_parent.addLayout()
+    layout = menu_parent.addLayout()  # type: ignore
 
     # Add a merge for any submenus (except toplevel)
     if merge:
@@ -292,10 +294,11 @@ def model_to_xml_layout(model, model_parent=None, menu_parent=None,  # noqa
         treeiter = model.iter_nth_child(model_parent, n_child)
 
         # Extract the menu item details.
-        name, comment, executable, categories, item_type, gicon, icon, desktop, expanded, \
-            show = model[treeiter][:]
+        name = model[treeiter][MenuEditor.COL_NAME]
+        item_type = model[treeiter][MenuEditor.COL_TYPE]
+        desktop = model[treeiter][MenuEditor.COL_FILENAME]
 
-        if item_type == MenuItemTypes.DIRECTORY:
+        if item_type == MenuItemTypes.DIRECTORY:  # type: ignore
             # Do not save duplicate directories.
             global processed_directories
             if desktop in processed_directories:
@@ -313,7 +316,7 @@ def model_to_xml_layout(model, model_parent=None, menu_parent=None,  # noqa
                 directory_name = util.getDirectoryName(desktop)
                 layout.addMenuname(directory_name)
 
-        elif item_type == MenuItemTypes.APPLICATION and desktop is not None:
+        elif item_type == MenuItemTypes.APPLICATION and desktop is not None:  # type: ignore
             try:
 
                 # According to the spec, desktop files may be located in
@@ -334,7 +337,7 @@ def model_to_xml_layout(model, model_parent=None, menu_parent=None,  # noqa
             except AttributeError:
                 pass
 
-        elif item_type == MenuItemTypes.SEPARATOR:
+        elif item_type == MenuItemTypes.SEPARATOR:  # type: ignore
             layout.addSeparator()
 
     # Add a merge for any new/unincluded menu items (except toplevel).
@@ -369,6 +372,10 @@ def treeview_to_xml(treeview):
     menu_name = MenuEditor.menu_name
     menu_file = MenuEditor.get_default_menu()
     merge_file = util.getSystemMenuPath(menu_file)
+
+    if menu_file is None:
+        return False
+
     filename = os.path.join(util.getUserMenusDirectory(), menu_file)
 
     # Create the menu XML
@@ -378,3 +385,5 @@ def treeview_to_xml(treeview):
 
     # Write the file.
     menu.write(filename)
+
+    return True
\ No newline at end of file
