@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2022 Sean Davis <sean@bluesabre.org>
+#   Copyright (C) 2012-2023 Sean Davis <sean@bluesabre.org>
 #   Copyright (C) 2017-2018 OmegaPhil <OmegaPhil@startmail.com>
 #
 #   This program is free software: you can redistribute it and/or modify it
@@ -16,6 +16,7 @@
 #   You should have received a copy of the GNU General Public License along
 #   with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import logging
 import os
 import re
 import subprocess
@@ -26,9 +27,10 @@ import shutil
 
 from locale import gettext as _
 
-from gi.repository import GLib, Gdk
+import gi
+gi.require_version('Gdk', '3.0')
+from gi.repository import GLib, Gdk  # type: ignore
 
-import logging
 logger = logging.getLogger('menulibre')
 
 old_psutil_format = isinstance(psutil.Process.username, property)
@@ -66,7 +68,7 @@ MenuItemKeys = (
     ("Exec", str, True, (0,)),
     ("Path", str, False, (0,)),
     ("Terminal", bool, False, (0,)),
-    ("Actions", list, False, (0,)),
+    ("Actions", str, False, (0,)),
     ("MimeType", list, False, (0,)),
     ("Categories", list, False, (0,)),
     ("Implements", list, False, (0,)),
@@ -80,11 +82,11 @@ MenuItemKeys = (
 def getRelatedKeys(menu_item_type, key_only=False):
     if isinstance(menu_item_type, str):
         if menu_item_type == "Application":
-            menu_item_type = MenuItemTypes.APPLICATION
+            menu_item_type = MenuItemTypes.APPLICATION  # type: ignore
         elif menu_item_type == "Link":
-            menu_item_type = MenuItemTypes.LINK
+            menu_item_type = MenuItemTypes.LINK  # type: ignore
         elif menu_item_type == "Directory":
-            menu_item_type = MenuItemTypes.DIRECTORY
+            menu_item_type = MenuItemTypes.DIRECTORY  # type: ignore
 
     results = []
     for tup in MenuItemKeys:
@@ -136,7 +138,7 @@ def getProcessList():
     """Return a list of unique process names for the current user."""
     username = getpass.getuser()
     try:
-        pids = psutil.get_pid_list()
+        pids = psutil.get_pid_list()  # type: ignore
     except AttributeError:
         pids = psutil.pids()
     processes = []
@@ -154,21 +156,25 @@ def getProcessList():
     return processes
 
 
-def getRelativeName(filename):
+def getRelativeName(filename: str):
     if filename.endswith('.desktop'):
         basename = filename.split('/applications/', 1)[1]
     elif filename.endswith('.directory'):
         basename = filename.split('/desktop-directories/', 1)[1]
+    else:
+        basename = ""
     return basename
 
 
-def getBasename(filename):
+def getBasename(filename: str):
     if filename.endswith('.desktop'):
         basename = filename.split('/applications/', 1)[1]
     elif filename.endswith('.directory'):
         basename = filename.split('/desktop-directories/', 1)[1]
         if basename.startswith("%s/" % getDefaultMenuName()):
             basename = filename.split("%s/" % getDefaultMenuName(), 1)[1]
+    else:
+        basename = ""
     return basename
 
 
@@ -193,7 +199,7 @@ def getDefaultMenuName():
     return prefix
 
 
-def getDefaultMenuPrefix(): # noqa
+def getDefaultMenuPrefix():  # noqa
     """Return the default menu prefix."""
     prefix = os.environ.get('XDG_MENU_PREFIX', '')
 
@@ -342,15 +348,15 @@ def mapDesktopEnvironmentDirectories():
         target_dir = os.path.join(user_dir, menu)
         try:
             os.makedirs(target_dir)
-        except:
+        except BaseException:
             pass
         target = os.path.join(target_dir, filename)
+        src = os.path.join(basedir, filename)
         if not os.path.exists(target):
-            src = os.path.join(basedir, filename)
             try:
                 logger.debug("copy %s to %s" % (src, target))
                 shutil.copy2(src, target)
-            except:
+            except BaseException:
                 logger.warning("Failed to copy %s to %s" % (src, target))
                 continue
 
@@ -359,7 +365,7 @@ def mapDesktopEnvironmentDirectories():
             if os.path.exists(symlink):
                 continue
             os.symlink(target, symlink)
-        except:
+        except BaseException:
             logger.warning("Failed to symlink %s to %s" % (src, target))
 
 
@@ -392,9 +398,10 @@ def unmapDesktopEnvironmentDirectories():
 
         try:
             os.remove(filename)
-        except:
+        except BaseException:
             logger.warning("Failed to remove symlink %s" % filename)
 
+
 def getUserDirectoriesDirectory():
     """Return the path to the user desktop-directories directory."""
     menu_dir = os.path.join(GLib.get_user_data_dir(), 'desktop-directories')
@@ -634,6 +641,8 @@ def getSaveFilename(name, filename, item_type, force_update=False):  # noqa
         elif item_type == 'Directory':
             path = getUserDirectoriesDirectory()
             ext = '.directory'
+        else:
+            path = ""
 
         basedir = os.path.dirname(os.path.join(path, basename))
         if not os.path.exists(basedir):
@@ -704,7 +713,10 @@ def check_keypress(event, keys):  # noqa
             return False
     if 'Escape' in keys:
         keys[keys.index('Escape')] = 'escape'
-    if Gdk.keyval_name(event.get_keyval()[1]).lower() not in keys:
+    keyval_name = Gdk.keyval_name(event.get_keyval()[1])
+    if keyval_name is None:
+        return False
+    if keyval_name.lower() not in keys:
         return False
 
     return True
@@ -744,7 +756,7 @@ def find_program(program):
     if len(program) == 0:
         return None
 
-    params = list(GLib.shell_parse_argv(program)[1])
+    params = list(GLib.shell_parse_argv(program)[1])  # type: ignore
     executable = params[0]
 
     if os.path.exists(executable):
@@ -756,6 +768,7 @@ def find_program(program):
 
     return None
 
+
 def validate_desktop_file(desktop_file):  # noqa
     """Validate a known-bad desktop file in the same way GMenu/glib does, to
     give a user real information about why certain files are broken."""
@@ -798,7 +811,7 @@ def validate_desktop_file(desktop_file):  # noqa
 
     # Type validation
     try:
-        type_key = keyfile.get_string(start_group,
+        type_key = keyfile.get_string(start_group,  # type: ignore
                                       GLib.KEY_FILE_DESKTOP_KEY_TYPE)
     except GLib.Error:
         # Translators: This error is displayed when a required key is
@@ -824,7 +837,7 @@ def validate_desktop_file(desktop_file):  # noqa
     # is not executable, the entry may be ignored (not be used in menus, for
     # example)."
     try:
-        try_exec = keyfile.get_string(start_group,
+        try_exec = keyfile.get_string(start_group,  # type: ignore
                                       GLib.KEY_FILE_DESKTOP_KEY_TRY_EXEC)
     except GLib.Error:
         pass
@@ -838,10 +851,10 @@ def validate_desktop_file(desktop_file):  # noqa
 
     # Validating executable
     try:
-        exec_key = keyfile.get_string(start_group,
+        exec_key = keyfile.get_string(start_group,  # type: ignore
                                       GLib.KEY_FILE_DESKTOP_KEY_EXEC)
     except GLib.Error:
-        return False # LP: #1788814, Exec key is not required
+        return False  # LP: #1788814, Exec key is not required
 
     try:
         if find_program(exec_key) is None:
@@ -853,7 +866,7 @@ def validate_desktop_file(desktop_file):  # noqa
                 % ('Exec', exec_key, e))
 
     if type_key == "Service":
-        return False # KDE services are not displayed in the menu
+        return False  # KDE services are not displayed in the menu
 
     # Translators: This error is displayed for a failing desktop file where
     # errors were detected but the file seems otherwise valid.
