@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2022 Sean Davis <sean@bluesabre.org>
+#   Copyright (C) 2012-2023 Sean Davis <sean@bluesabre.org>
 #
 #   This program is free software: you can redistribute it and/or modify it
 #   under the terms of the GNU General Public License version 3, as published
@@ -19,6 +19,6 @@
 
 while keeping its api constant'''
 # lint:disable
-from . helpers import get_builder, set_up_logging, show_uri  # noqa
+from . helpers import set_up_logging, show_uri  # noqa
 from . menulibreconfig import get_version  # noqa
 # lint:enable
